#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

#endregion // Namespaces

namespace AcaNetIntroCs
{
  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class Commands1b
  {
    /// <summary>
    /// Demonstrate exception handling.
    /// </summary>
    [CommandMethod( "TestException" )]
    static public void TestException()
    {
      double result = 0.0;
      try
      {
        // Convertor is in Autodesk.AutoCAD.Runtime,
        // so it would raise Autodesk.AutoCAD.Runtime.Exception
        result = Converter.StringToDistance( "Will Fail" );
      }
      catch( System.Exception e )
      {
        // Since Autodesk.AutoCAD.Runtime.Exception is derived from System.Exception,
        // this block fill catch *any* exception
        App.MsgBox( e.GetType().ToString() + ": " + e.Message + "\n" + e.ToString() );
        result = double.NaN;
      }
      finally
      {
        // This block is executed *always*
        App.MsgBox("Distance: " + result.ToString() );
      }
    }

    /// <summary>
    /// Demonstrate selection using GetEntity() and transaction.
    /// </summary>
    [CommandMethod( "TestSelect", CommandFlags.Modal | CommandFlags.UsePickSet )]
    static public void TestSelect()
    {
      Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
      PromptEntityOptions optEnt = new PromptEntityOptions( "Select an AEC entity" );
      optEnt.SetRejectMessage( "Selected entity is NOT a AEC entity, please try again... " );
      optEnt.AddAllowedClass( typeof( Geo ), false ); // Geo is the base class of AEC entities

      PromptEntityResult resEnt = ed.GetEntity( optEnt );
      if( PromptStatus.OK != resEnt.Status )
      {
        ed.WriteMessage( "Selection error - aborting" );
      }
      else
      {
        // Even to open the object for read-only we need a transaction.
        // Here we explicitly dispose of the transaction after use.
        // See below for an equivalent and more succint syntax making 
        // use of the 'using' statement.
        Transaction tr = AcadApp.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction();
        try
        {
          AcObject obj = tr.GetObject( resEnt.ObjectId, OpenMode.ForRead );
          App.MsgBox( "You selected a " + obj.GetRXClass().Name );
          tr.Commit();
        }
        catch
        {
          //tr.Abort(); // not necessary, since Abort is called from Dispose if no Commit
        }
        finally
        {
          tr.Dispose();
        }
      }
    }

    /// <summary>
    /// Demonstrate selection set using GetSelection() and edit wall entities.
    /// </summary>
    [CommandMethod( "TestRotate", CommandFlags.Modal | CommandFlags.UsePickSet )]
    static public void TestRotate()
    {
      Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
      PromptSelectionOptions optSs = new PromptSelectionOptions(); // optional selection options
      optSs.MessageForAdding = "Select some walls, please";
      TypedValue[] selRb = new TypedValue[1]; // optional selection filter
      selRb[0] = new TypedValue( (int) DxfCode.Start, "AEC_WALL" );
      SelectionFilter filter = new SelectionFilter( selRb );
      PromptSelectionResult resSs = ed.GetSelection( optSs, filter ); // select
      if( PromptStatus.OK != resSs.Status )
      {
        ed.WriteMessage( "Selection error - aborting" );
      }
      else
      {
        try
        {
          using( Transaction tr = AcadApp.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction() )
          {
            // Loop selection and change
            foreach( SelectedObject obj in resSs.Value )
            {
              Wall wall = tr.GetObject( obj.ObjectId, OpenMode.ForWrite ) as Wall;
              string s = "\nWall " + wall.Handle.ToString();
              ed.WriteMessage( s + " OLD rotation = " + wall.Rotation.ToString() );
              wall.Rotation += System.Math.PI / 20; // Add 9 degrees to the rotation
              ed.WriteMessage( s + " NEW rotation = " + wall.Rotation.ToString() );
            }
            tr.Commit();
          }
        }
        catch( Exception e )
        {
          App.MsgBox( e.GetType().ToString() + ": " + e.Message + "\n" + e.ToString() );
        }
      }
    }

    /// <summary>
    /// Helper to list properties using reflection.
    /// </summary>
    /// <param name="ent">Input entity</param>
    /// <returns>Concatenation of all entity property names and values</returns>
    static string ListProps( AcEntity ent )
    {
      string str = string.Empty;
      BindingFlags bf = BindingFlags.Instance | BindingFlags.Public;
      MemberInfo[] memInfoArray = ent.GetType().FindMembers( MemberTypes.Property, bf, null, null );

      foreach( MemberInfo memInfo in memInfoArray )
      {
        try
        {
          object o = ent.GetType().InvokeMember( memInfo.Name, BindingFlags.GetProperty, null, ent, null );
          str += "\nProp." + memInfo.Name + " = " + o.ToString();
        }
        catch
        {
          str += "\nProp." + memInfo.Name + " = [exception]";
        }
      }
      return str;
    }

    /// <summary>
    /// Use .NET Reflection to get style name and all properties.
    /// </summary>
    [CommandMethod( "TestReflection", CommandFlags.Modal | CommandFlags.UsePickSet )]
    static public void TestReflection()
    {
      Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
      PromptSelectionResult resSs = ed.GetSelection(); // select
      if( PromptStatus.OK != resSs.Status )
      {
        ed.WriteMessage( "Selection error - aborting" );
      }
      else
      {
        try
        {
          using( Transaction tr = AcadApp.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction() )
          {
            ObjectId[] ids = resSs.Value.GetObjectIds();
            foreach( ObjectId id in ids )
            {
              AcEntity ent = tr.GetObject( id, OpenMode.ForRead ) as AcEntity;
              string msg = "Selected Entity is " + ent.GetType().ToString() + "\n";
              // Use reflection ("late binding") to check if StyleId supported
              try
              {
                object styleId = ent.GetType().InvokeMember( "StyleId", BindingFlags.GetProperty, null, ent, null );
                DictionaryRecord rec = tr.GetObject( (ObjectId) styleId, OpenMode.ForRead ) as DictionaryRecord;
                msg += "\nStyle name = " + rec.Name + "\n";
              }
              catch
              {
                msg += "\nNO StyleId property supported.\n";
              }
              msg += ListProps( ent ); // get all properties defined on the type
              App.MsgBox( msg ); // display the result
            }
            tr.Commit();
          }
        }
        catch( Exception e )
        {
          App.MsgBox( e.GetType().ToString() + ": " + e.Message + "\n" + e.ToString() );
        }
      }
    }
  }
}
