#region Header
//
// .NET Sample
//
// Copyright (C) 2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Collections.Specialized; // StringCollection
using System.Diagnostics;
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.ApplicationServices;
using Autodesk.Aec.PropertyData;
using Autodesk.Aec.PropertyData.DatabaseServices;
using Autodesk.Aec.Arch.ApplicationServices;
using Autodesk.Aec.Arch.DatabaseServices;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

#endregion // Namespaces

namespace AcaNetIntroCs
{
  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class Commands4
  {
    /// <summary>
    /// Helper method to select a wall, using standard AutoCAD.NET functionality.
    /// </summary>
    /// <param name="wallId">Return selected wall object id</param>
    /// <param name="ed">Editor</param>
    /// <returns>True on successful selection</returns>
    public static bool SelectWall( out ObjectId wallId, Editor ed )
    {
      PromptEntityOptions optEnt = new PromptEntityOptions( "Please select a wall: " );
      optEnt.SetRejectMessage( "Selected entity is not a wall, please try again..." );
      optEnt.AddAllowedClass( typeof( Wall ), true );
      PromptEntityResult resEnt = ed.GetEntity( optEnt );
      bool rc = (PromptStatus.OK == resEnt.Status);
      wallId = rc ? resEnt.ObjectId : ObjectId.Null;
      return rc;
    }

    /// <summary>
    /// List all properties from all property sets attached to all entities.
    /// </summary>
    [CommandMethod( "TestDispProps" )]
    static public void TestDispProps()
    {
      try
      {
        Database db = HostApplicationServices.WorkingDatabase;
        Editor ed = AcadApp.DocumentManager.MdiActiveDocument.Editor;
        using( Transaction tr = db.TransactionManager.StartTransaction() )
        {
          ObjectId wallId;
          if( SelectWall( out wallId, ed ) )
          {
            Wall wall = tr.GetObject( wallId, OpenMode.ForRead ) as Wall;
            ed.WriteMessage( "You selected a " + wall.GetRXClass().Name + "\n" );
            WallStyle wallStyle = tr.GetObject( wall.StyleId, OpenMode.ForRead ) as WallStyle;
            App.MsgBox( string.Format( "Got the wall {0} and its style object {1}.", wall.Handle.ToString(), wallStyle.Name ) );
            // get the display rep manager - center of the display system. 
            DisplayRepresentationManager dispRepMgr = new DisplayRepresentationManager( db );
            // get all DispReps for wall used in the current viewport and loop them
            ObjectIdCollection ids = dispRepMgr.GetDisplayRepresentationIdsFromCurrentViewport( wall.GetRXClass() );
            //ed.WriteMessage( "wall disp rep count = " + ids.Count );
            DisplayRepresentation dispRep;
            string s;
            foreach( ObjectId id in ids )
            {
              dispRep = tr.GetObject( id, OpenMode.ForRead ) as DisplayRepresentation;
              s = "--  Display Properties Override  --\n\n";
              s += "Display Representation Name: " + dispRep.Name + ", Display Name: " + dispRep.DisplayName + "\n";
              RXClass rcProps = dispRep.DisplayPropertiesClass;
              s += "Display Props Class: " + rcProps.Name + "\n";
              //
              // find display properties override
              //
              DisplayProperties dps;
              // (1) first check at element level.
              ObjectId dpId = dispRep.FindDisplayPropertiesOverride( wall );
              if( !dpId.IsNull )
              {
                s += "ELEMENT OVERRIDE: ";
              }
              else
              {
                // (2) check for style override
                dpId = dispRep.FindDisplayPropertiesOverride( wallStyle );
                if( !dpId.IsNull )
                {
                  s += "STYLE OVERRIDE: ";
                }
                else
                {
                  // (3) if not at element or style, then must be at drawing default. 
                  dpId = dispRep.DefaultDisplayPropertiesId;
                  if( !dpId.IsNull )
                  {
                    s += "DRAWING DEFAULT: ";
                  }
                }
              }
              Debug.Assert( !dpId.IsNull, "expected non-null dispprops object id" );
              dps = tr.GetObject( dpId, OpenMode.ForRead ) as DisplayProperties;
              s += dps.Name + " Display Name:" + dps.DisplayName + "\n";
              App.MsgBox( s );

              s = "--  Display Components --\n\n";
              // show a few display component
              StringCollection sc = new StringCollection(); // must be intialised!
              DisplayComponent[] components = dps.GetDisplayComponents( out sc );
              if( 0 < components.Length )
              {
                for( int i = 0; i < components.Length; ++i )
                {
                  DisplayComponent dispComp = components[i];
                  string name = sc[i];
                  // Hatch or entity component? 
                  if( dispComp is DisplayComponentHatch )
                  {
                    DisplayComponentHatch compHatch = dispComp as DisplayComponentHatch;
                    // list a few properties 
                    s += "HATCH Component: " + name;
                    s += ": Visibility=" + compHatch.IsVisible.ToString();
                    s += ", Color=" + compHatch.ColorIndex.ToString();
                    s += ", Pattern=" + compHatch.PatternName + "\n";
                  }
                  else if( dispComp is DisplayComponentEntity )
                  {
                    DisplayComponentEntity compEnt = dispComp as DisplayComponentEntity;
                    // list a few properties 
                    s += "ENTITY Component: " + name;
                    s += ": Visibility=" + compEnt.IsVisible.ToString();
                    s += ", Color=" + compEnt.ColorIndex.ToString() + "\n";
                  }
                }
              }
              App.MsgBox( s );
            }
          }
          tr.Commit();
        }
      }
      catch( Exception ex )
      {
        App.MsgBox( "Exception: " + ex.Message );
      }
    }
  }
}
