#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.ApplicationServices;
using Autodesk.Aec.Arch.ApplicationServices;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

// To resolve Application ambiguities
using AcadNETApp = Autodesk.AutoCAD.ApplicationServices.Application;

using Autodesk.AutoCAD.Interop;
using Autodesk.AutoCAD.Interop.Common;
using Autodesk.AEC.Interop.Base;
using Autodesk.AEC.Interop.ArchBase;
using Autodesk.AEC.Interop.UIBase;

#endregion // Namespaces

namespace AcaNetIntroCs
{
  /// <summary>
  /// A class containing commands.
  /// </summary>
  public class Commands5
  {
    /// <summary>
    /// Check which display properties a wall uses for all current dispreps and list some display components.
    /// This shows mixed usage of native .NET and COM Interop API (display system is exposed in the former as well since 2008).
    /// </summary>
    [CommandMethod( "TestComDispProps" )]
    static public void TestComDispProps()
    {
      Editor ed = AcadNETApp.DocumentManager.MdiActiveDocument.Editor;
      // Minimum error checking!
      //// First use *native .NET* to select a wall and get the wall style Id
      //PromptEntityOptions optEnt = new PromptEntityOptions( "Please select a wall: " );
      //optEnt.SetRejectMessage( "\nSelected entity is not a wall, please try again..." );
      //optEnt.AddAllowedClass( typeof(Wall), true );
      //PromptEntityResult resEnt = ed.GetEntity( optEnt );
      //if( PromptStatus.OK != resEnt.Status )

      ObjectId wallId;
      if( !Commands4.SelectWall( out wallId, ed ) )
      {
        ed.WriteMessage( "Selection error - aborting" );
        return;
      }
      // AutoCAD ActiveX App and Doc (for convenience name it ThisDrawing if porting VBA code)
      AcadApplication ThisApp = AcadNETApp.AcadApplication as AcadApplication;
      AcadDocument ThisDrawing = ThisApp.ActiveDocument;
      // Since we later need *ActiveX* Wall object, the following is the way to obtain one from ObjectId:
      AecWall axWall = ThisDrawing.ObjectIdToObject( (int) wallId.OldIdPtr ) as AecWall;
      AecWallStyle axWallStyle = axWall.Style;
      App.MsgBox( "Got ActiveX Wall and WallStyle objects: " + axWall.Handle + ", " + axWallStyle.Name );
      // AEC ActiveX app and doc (enough for dispreps - we don't need ACA app and doc)
      AecBaseApplication app = ThisApp.GetInterfaceObject( "AecX.AecBaseApplication.5.7" ) as AecBaseApplication;
      app.Init( ThisApp );
      AecBaseDocument doc = app.ActiveDocument as AecBaseDocument;
      // get all DispReps used in the current viewport and loop them all
      // Dim dispset As AecDisplaySet = doc.DisplaySets.ActiveSet(ThisDrawing.ActiveViewport)
      // Dim dispreps As AecDisplayRepresentations = dispset.DisplayRepresentations
      AecDisplayRepresentations dispreps = doc.DisplaySets.get_ActiveSet( ThisDrawing.ActiveViewport as AcadObject ).DisplayRepresentations;
      foreach( AecDisplayRepresentation disprep in dispreps )
      {
        // We are interested ONLY in those drawing for walls
        if( disprep.ClassName == "Wall" )
        {
          string msg = disprep.DisplayName + " DispRep " + disprep.Name + " uses ";
          AecDisplayProperties wallDPs;
          // Get corresponding Display Properties object
          try
          {
            // check for OBJECT OVERRIDE
            wallDPs = disprep.get_OverrideDisplayProperties( axWall as AcadObject );
            msg += "OBJECT OVERRIDE:";
          }
          catch
          {
            try
            {
              // check for STYLE OVERRIDE
              wallDPs = disprep.get_OverrideDisplayProperties( axWallStyle as AcadObject );
              msg += "STYLE OVERRIDE:";
            }
            catch
            {
              try
              {
                // must be DWG DEFAULT
                wallDPs = disprep.DefaultDisplayProperties;
                msg += "DWG DEFAULT:";
              }
              catch
              {
                App.MsgBox( "No default DispReps??? - aborting..." );
                return;
              }
            }
          }
          // List a few Display Components
          AecDisplayComponents dispcomps;
          dispcomps = wallDPs.DisplayComponents;
          foreach( AecDisplayComponent dispcomp in dispcomps )
          {
            // First must check the most-derived class (note: Hatch is derived from Entity)
            if( dispcomp is AecDisplayComponentHatch )
            {
              AecDisplayComponentHatch comphatch = dispcomp as AecDisplayComponentHatch;
              // NOTE: TrueColor.ColorIndex will fail if RGB
              // or .HatchPattern may fail if user hatch, but keep it simple here...
              // List only a few
              msg += "\n  HATCH component " + dispcomp.Name
                + ": Vis=" + comphatch.Visible
                + ", Col=" + comphatch.TrueColor.ColorIndex
                + ", Pat=" + comphatch.PatternName;
            }
            else if( dispcomp is AecDisplayComponentEntity )
            {
              AecDisplayComponentEntity compent = dispcomp as AecDisplayComponentEntity;
              // NOTE: TrueColor.ColorIndex will fail if RGB
              // List only a few
              msg += "\n  CURVES component " + compent.Name
                + ": Vis=" + compent.Visible
                + ", Col=" + compent.TrueColor.ColorIndex;
            }
          }
          App.MsgBox( msg );
        }
      }
    }
  }
}
