// $Header: //SandBox/awej/Content/AllUsers/RcpSamples/Source/SampRcp_CPP/Src/SampRcpMisc.cpp#1 $
// $Author: awej $ $DateTime: 2004/05/05 14:47:54 $ $Change: 11180 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

#include "stdafx.h"
#include "SampRcpMisc.h"
#include "AecRcpBase/AecRecipeInfoAcc.h"
#include "AecRcpBase/AecRcpCompBldrCurSpace.h"
#include "AecRcpBase/AecRcpIdStr.h"
#include "AecRcpBase/AecRcpJigUtils.h"

/****************************************************************************
**
**  SampRcpMisc::registerFuncs
**      Here we associate a name with a function.  The name "Rectangle" is what
**  is referenced from the XML RecipeSpec with the following construct:
**
**		<Implementation>
**			<ObjectARX>
**				<AppName>SampRcpCpp40</AppName>
**				<RcpFunc>Rectangle</RcpFunc>
**			</ObjectARX>
**		</Implementation>
**
**  RcpFunc names only need to be unique within a given application because they
**  are namespaced.
**
**
*************************************/

void
SampRcpMisc::registerFuncs()
{
    addFunc(_DNT("Rectangle"),  &rectangle);
}

/****************************************************************************
**
**  SampRcpMisc::rectangle
**      This is the "HelloWorld!" of Recipes.
**
**  Recipes usually follow a common pattern:
**  1)  Retrieve data from various places (DB, PropPalette, XML, etc)
**  2)  Generate the necessary geometry into a CompBldr object
**      a)  If your entity is "profile" based (meaning it has a Polyline boundary),
**          it often makes sense to have code separated into a function that generates
**          the raw points and bulges, and code that generates the Polyline from those
**          calculated points and bulges.
**      b)  If you visually want to see the entity take shape as the user picks points,
**          (as in a Joist component), then you can write a JigEnt to do all the calculations
**          and to plug into a Jig to provide the user feedback.
**  3)  Place the component using a Jig
**
*************************************/

RcpStatus
SampRcpMisc::rectangle()
{
        // This accessor object gives us pointers to all the data structures we might need.
        // From this object (AecRecipeInfoAcc), we can get pointers to data from the Access
        // database, the XML RecipeSpec, and the PropertyPalette.
    AecRecipeInfoAcc rcpInfo;
    if (rcpInfo.isValid() == false)
        return rcpError();

        // get the values for SIZE_X and SIZE_Y from the currently selected row of the database.
        // NOTE: most distance values are Unit-Dependent (meaning they need to be converted from
        // inches to millimeters, cm to m, etc.)  There are signatures without the "UnitDep" suffix
        // that do not do any conversion, if that is truly what you need (Like if the double value
        // represents an angle).
    double lenX, lenY;
    if ((rcpInfo.dbData()->getFieldValueUnitDep(_DNT("SIZE_X"), lenX, rcpInfo.database()) != Acad::eOk) ||
        (rcpInfo.dbData()->getFieldValueUnitDep(_DNT("SIZE_Y"), lenY, rcpInfo.database()) != Acad::eOk))
        return rcpError();

        // build the component geometry using a "Component Builder".  A CompBldr manages some of the
        // low-level details of packaging together entities.  AecRcpCompBldrBlkDef will add the entities
        // to an Anonymous block definition.  AecRcpCompBldrCurSpace will add the entities to either
        // ModelSpace or PaperSpace (whichever is current).  Which you use depends on the type of component
        // you are building.  Usually, everything will be a block unless it is something that can be
        // easily stretched and remain a valid component.
    AecRcpCompBldrBlkDef compBldr(rcpInfo);
    if (compBldr.start() != Acad::eOk)
        return rcpError();

        // we are making a simple rectangle, so we just need to make the 4 points.
    AcGePoint2dArray pts;
    AcGePoint2d tmpPt;

        // origin in center, start at lower left corner
    tmpPt.x = -(lenX / 2.0);
    tmpPt.y = -(lenY / 2.0);
    pts.append(tmpPt);

    tmpPt.x += lenX;
    pts.append(tmpPt);

    tmpPt.y += lenY;
    pts.append(tmpPt);

    tmpPt.x -= lenX;
    pts.append(tmpPt);

        // Allocate a new Polyline and set its properties for Layer/Color/Linetype, etc.
        // We set some default properties and then ask the XML RecipeSpec if it has specific
        // properties we need to use.
    AcDbPolyline* pline = new AcDbPolyline;
    compBldr.setToDefaultProps(pline);      // these are different depending on what type of CompBldr it is
    rcpInfo.rcData()->getItem_EntityProps(AecRcpIdStr::boundary, pline);    // look for a specific section called "boundary"
    compBldr.addPolyline(pline, pts, true); // add this polyline to the CompBldr

        // The CompBldr usually gets sent more than a single entity.  When you call ::addPolyline(),
        // ::addHatch(), or ::addToDb(), it accepts responsibility for the memory we allocated for the
        // AcDbPolyline.  The Polyline is added through a AcTransaction.  If the entity is successfully
        // added to the Database, then the Transaction will close() it or cancel() it when the transaction
        // is ended or aborted.  If the entity was not correctly added to the database, then delete will
        // be called on that entity.  So, once we have passed it off to the CompBldr, it is no longer our
        // repsonsibility, but it is still open for our use until the call to compBldr.endOrAbort().
    if (compBldr.endOrAbort() == Acad::eOk)
        return rcpResult(AecRcpJigUtils::stampIt(compBldr));
    else
        return rcpError();
}
