#region Header
// Rasterize.Commands.cs - Aec StreamVector sample to rasterize an entity
//
// Written 2008-08-08 by Jeremy Tammik, Autodesk Inc.
#endregion // Header

#region Copyright
// Copyright (c) 2008 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
#endregion // Copyright

#region Namespaces
using System;
using System.Drawing;
using System.Drawing.Imaging;
using WinForms = System.Windows.Forms;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Runtime;

using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Geometry;
//using Autodesk.Aec.Arch.DatabaseServices;

using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;
using DBObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using Database = Autodesk.AutoCAD.DatabaseServices.Database;
using TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager;
using Transaction = Autodesk.AutoCAD.DatabaseServices.Transaction;
using OpenMode = Autodesk.AutoCAD.DatabaseServices.OpenMode;
using LineWeight = Autodesk.AutoCAD.DatabaseServices.LineWeight;
using BlockReference = Autodesk.AutoCAD.DatabaseServices.BlockReference;
using Entity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecModeler = Autodesk.Aec.Modeler;
#endregion // Namespaces

[assembly: CommandClass( typeof( Rasterize.Commands ) )]

namespace Rasterize
{
  public class Commands
  {
    #region Util
    /// <summary>
    /// Return the current AutoCAD editor.
    /// </summary>
    static private Editor GetEditor()
    {
      return Application.DocumentManager.MdiActiveDocument.Editor;
    }

    /// <summary>
    /// Return the current AutoCAD active database.
    /// </summary>
    static private Database GetDatabase()
    {
      return Application.DocumentManager.MdiActiveDocument.Database;
    }

    /// <summary>
    /// Prompt the user to select an entity.
    /// </summary>
    /// <param name="type">Required System.Type or null</param>
    /// <param name="exactMatch">Exact match, or derived classes allowed?</param>
    /// <param name="tips">Prompt string</param>
    /// <returns>Selected entity object id or null</returns>
    static ObjectId PickObject( Type type, bool exactMatch, string tips )
    {
      Editor editor = GetEditor();
      PromptEntityOptions options = null;
      PromptEntityResult result;
      if( type != null )
      {
        options = new PromptEntityOptions( tips );
        options.SetRejectMessage( "This type of object is not allowed.\n" );
        options.AddAllowedClass( type, exactMatch );
        result = editor.GetEntity( options );
      }
      else
      {
        result = editor.GetEntity( tips );
      }
      return ( result.Status == PromptStatus.OK && result.ObjectId.IsValid )
        ? result.ObjectId
        : ObjectId.Null;
    }

    static ObjectId PickObject( Type type, bool exactMatch )
    {
      return PickObject( type, exactMatch, "Please pick an object or return" );
    }

    static ObjectIdCollection PickObjectSet( string tips )
    {
      Editor editor = GetEditor();
      PromptSelectionOptions options = new PromptSelectionOptions();
      options.MessageForAdding = tips;
      options.MessageForRemoval = "Remove objects from selection";
      options.AllowDuplicates = false;
      options.RejectObjectsFromNonCurrentSpace = true;
      ObjectIdCollection ids = new ObjectIdCollection();
      PromptSelectionResult result = editor.GetSelection( options );
      if( result.Status == PromptStatus.OK )
      {
        SelectionSet set = result.Value;
        foreach( SelectedObject obj in set )
        {
          ids.Add( obj.ObjectId );
        }
      }
      return ids;
    }

    static ObjectIdCollection PickObjectSet()
    {
      return PickObjectSet( "Add objects to selection" );
    }
    #endregion // Util

    #region Scaler
    /// <summary>
    /// Helper class to convert from 3D AutoCAD WCS coordinate space to 2D bitmap pixel coordinate.
    /// </summary>
    class Scaler
    {
      Point3d _min_point;
      double _factor_x;
      double _factor_y;

      public Scaler( BoundBox3d sourcebox, Size targetsize )
      {
        _min_point = sourcebox.MinPoint;
        _factor_x = targetsize.Width / sourcebox.Width;
        _factor_y = targetsize.Height / sourcebox.Depth; // sourcebox.Height is in Z direction!
      }

      /// <summary>
      /// Scale a 3D AutoCAD WCS coordinate space point to 2D bitmap pixel coordinates.
      /// </summary>
      /// <param name="p">3D AutoCAD WCS coordinate space point</param>
      /// <returns>2D bitmap pixel coordinates</returns>
      public Point Scale( Point3d p )
      {
        return new Point( 
          ( int ) ( _factor_x * ( p.X - _min_point.X ) ),
          ( int ) ( _factor_y * ( p.Y - _min_point.Y ) ) );
      }
    }
    #endregion // Scaler

    [CommandMethod( "Rasterize" )]
    static public void Rasterize()
    {
      string prompt = "Rasterize demonstrates the ability of StreamVector"
        + " to break an entity into primitive lines and generate a bitmap"
        + " image from them. Please select some objects to generate bitmap.\n";
      Editor ed = GetEditor();
      ed.WriteMessage( prompt );

      ObjectIdCollection ids = PickObjectSet();
      if( 0 == ids.Count )
      {
        ed.WriteMessage( "No objects selected.\n" );
      }
      else
      {
        //
        // set up a StreamVector instance to collect lines:
        //
        Database db = GetDatabase();
        StreamVector stream = new StreamVector( db );
        StreamExtent stream2 = new StreamExtent( db );
        TransactionManager tm = db.TransactionManager;
        using( Transaction trans = tm.StartTransaction() )
        {
          ObjectId dcId = DictionaryDisplayConfiguration.GetStandardDisplayConfiguration( db );

          // set up the display parameters
          stream.PushDisplayParameters( dcId, trans );
          stream2.PushDisplayParameters( dcId, trans );

          foreach( ObjectId id in ids )
          {
            Entity ent = trans.GetObject( id, OpenMode.ForRead ) as Entity;
            if( null != ent )
            {
              // stream the object
              stream.Stream( ent );
              stream2.Stream( ent );
            }
          }
          stream.PopDisplayParameters();
          stream2.PopDisplayParameters();

          trans.Commit(); // always commit, for better performance
        }
        //
        // process the collected lines:
        // 
        WinForms.SaveFileDialog dlg = new WinForms.SaveFileDialog();
        dlg.Title = "Please select bitmap file to save";
        dlg.Filter = "PNG Bitmap Files (*.png)|*.png|All files (*)|*";
        dlg.CheckPathExists = false;
        if( WinForms.DialogResult.OK == dlg.ShowDialog() )
        {
          int width = 256;
          int height = 256;
          using( Bitmap bitmap = new Bitmap( width, height ) )
          {
            BoundBox3d box = stream2.Extents;
            Size size = new Size( width, height );
            Scaler scaler = new Scaler( box, size );
            Graphics graphics = Graphics.FromImage( bitmap );
            Pen pen = new Pen( Color.Black );
            foreach( LineSegment3d line in stream.Lines )
            {
              graphics.DrawLine( pen, scaler.Scale( line.StartPoint ), scaler.Scale( line.EndPoint ) );
            }
            bitmap.Save( dlg.FileName, ImageFormat.Png );
          }
        }
      }
    }
  }
}
